@@ -262,7 +262,7 @@ static struct artwork_source artwork_item_source[] =
     {
       .name = "stream",
       .handler = source_item_artwork_url_get,
-      .data_kinds = (1 << DATA_KIND_HTTP),
+      .data_kinds = (1 << DATA_KIND_HTTP) | (1 << DATA_KIND_PIPE),
       .media_kinds = MEDIA_KIND_MUSIC,
       .cache = NEVER,
     },
@@ -439,7 +439,7 @@ artwork_read_byurl(struct evbuffer *evbuf, const char *url)
   client.input_headers = kv;
   client.input_body = evbuf;
 
-  ret = http_client_request(&client);
+  ret = http_client_request(&client, NULL);
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_ART, "Request to '%s' failed with return value %d\n", url, ret);
@@ -851,7 +851,13 @@ parent_dir_image_find(char *out_path, size_t len, const char *dir)
       DPRINTF(E_LOG, L_ART, "Could not find parent dir name (%s)\n", path);
       return -1;
     }
-  strcpy(parentdir, ptr + 1);
+
+  ret = snprintf(parentdir, sizeof(parentdir), "%s", ptr + 1);
+  if ((ret < 0) || (ret >= sizeof(parentdir)))
+    {
+      DPRINTF(E_LOG, L_ART, "Impossible error occured in parent_dir_image_find(), cause was: %s\n", ptr + 1);
+      return -1;
+    }
 
   path_len = strlen(path);
   nextensions = ARRAY_SIZE(cover_extension);
@@ -1282,7 +1288,7 @@ online_source_search(struct online_source *src, struct artwork_ctx *ctx)
   client.url = url;
   client.output_headers = &output_headers;
 
-  ret = http_client_request(&client);
+  ret = http_client_request(&client, NULL);
   keyval_clear(&output_headers);
   if (ret < 0 || client.response_code != HTTP_OK)
     {
@@ -1377,7 +1383,7 @@ source_group_dir_get(struct artwork_ctx *ctx)
       return ART_E_ERROR;
     }
 
-  while (((ret = db_query_fetch_string(&qp, &dir)) == 0) && (dir))
+  while (((ret = db_query_fetch_string(&dir, &qp)) == 0) && (dir))
     {
       /* The db query may return non-directories (eg if item is an internet stream or Spotify) */
       if (access(dir, F_OK) < 0)
@@ -1549,10 +1555,7 @@ source_item_pipe_get(struct artwork_ctx *ctx)
 
   queue_item = db_queue_fetch_byfileid(ctx->id);
   if (!queue_item || !queue_item->artwork_url || strncmp(queue_item->artwork_url, proto, strlen(proto)) != 0)
-    {
-      free_queue_item(queue_item, 0);
-      return ART_E_NONE;
-    }
+    goto notfound;
 
   path = queue_item->artwork_url + strlen(proto);
 
@@ -1560,15 +1563,16 @@ source_item_pipe_get(struct artwork_ctx *ctx)
   // been updated yet. In that case just stop now.
   ret = access(path, F_OK);
   if (ret != 0)
-    {
-      return ART_E_NONE;
-    }
+    goto notfound;
 
   snprintf(ctx->path, sizeof(ctx->path), "%s", path);
 
   free_queue_item(queue_item, 0);
-
   return artwork_get(ctx->evbuf, ctx->path, NULL, false, ctx->data_kind, ctx->req_params);
+
+ notfound:
+  free_queue_item(queue_item, 0);
+  return ART_E_NONE;
 }
 
 static int
@@ -1720,7 +1724,7 @@ source_item_ownpl_get(struct artwork_ctx *ctx)
   mfi_path = ctx->dbmfi->path;
 
   format = ART_E_NONE;
-  while (((ret = db_query_fetch_pl(&qp, &dbpli)) == 0) && (dbpli.id) && (format == ART_E_NONE))
+  while (((ret = db_query_fetch_pl(&dbpli, &qp)) == 0) && (dbpli.id) && (format == ART_E_NONE))
     {
       if (!dbpli.path)
 	continue;
@@ -1768,7 +1772,7 @@ process_items(struct artwork_ctx *ctx, int item_mode)
       return -1;
     }
 
-  while (((ret = db_query_fetch_file(&ctx->qp, &dbmfi)) == 0) && (dbmfi.id))
+  while ((ret = db_query_fetch_file(&dbmfi, &ctx->qp)) == 0)
     {
       // Save the first songalbumid, might need it for process_group() if this search doesn't give anything
       if (!ctx->persistentid)
@@ -1861,7 +1865,7 @@ process_group(struct artwork_ctx *ctx)
       goto invalid_group;
     }
 
-  is_valid = (db_query_fetch_file(&ctx->qp, &dbmfi) == 0 && dbmfi.id && strcmp(dbmfi.album, CFG_NAME_UNKNOWN_ALBUM) != 0 && strcmp(dbmfi.album_artist, CFG_NAME_UNKNOWN_ARTIST) != 0);
+  is_valid = (db_query_fetch_file(&dbmfi, &ctx->qp) == 0 && strcmp(dbmfi.album, CFG_NAME_UNKNOWN_ALBUM) != 0 && strcmp(dbmfi.album_artist, CFG_NAME_UNKNOWN_ARTIST) != 0);
   db_query_end(&ctx->qp);
   if (!is_valid)
     {
