@@ -48,17 +48,14 @@ events for proceeding are activated directly.
 */
 
 #include <pthread.h>
+#include <assert.h>
 
 #include "librespot-c-internal.h"
 #include "commands.h"
 #include "connection.h"
 #include "channel.h"
 
-/* TODO list
-
- - protect against DOS
-*/
-
+// #define DEBUG_DISCONNECT 1
 
 /* -------------------------------- Globals --------------------------------- */
 
@@ -77,6 +74,9 @@ static struct commands_base *sp_cmdbase;
 
 static struct timeval sp_response_timeout_tv = { SP_AP_TIMEOUT_SECS, 0 };
 
+#ifdef DEBUG_DISCONNECT
+static int debug_disconnect_counter;
+#endif
 
 // Forwards
 static int
@@ -96,6 +96,8 @@ session_free(struct sp_session *session)
   ap_disconnect(&session->conn);
 
   event_free(session->continue_ev);
+
+  free(session->ap_avoid);
   free(session);
 }
 
@@ -212,7 +214,7 @@ session_return(struct sp_session *session, enum sp_error err)
     {
       // track_write() completed, close the write end which means reader will
       // get an EOF
-      if (channel && channel->is_writing && err == SP_OK_DONE)
+      if (channel && channel->state == SP_CHANNEL_STATE_PLAYING && err == SP_OK_DONE)
 	channel_stop(channel);
       return;
     }
@@ -220,15 +222,12 @@ session_return(struct sp_session *session, enum sp_error err)
   commands_exec_end(sp_cmdbase, err);
 }
 
-// Rolls back from an error situation. If it is a failed login then the session
-// will be closed, but if it just a connection timeout we keep the session, but
-// drop the ongoing download.
+// Disconnects after an error situation. If it is a failed login then the
+// session, otherwise we end download and disconnect.
 static void
 session_error(struct sp_session *session, enum sp_error err)
 {
-  struct sp_channel *channel = session->now_streaming_channel;
-
-  sp_cb.logmsg("Session error: %d\n", err);
+  sp_cb.logmsg("Session error: %d (occurred before msg %d, queue %d)\n", err, session->msg_type_next, session->msg_type_queued);
 
   session_return(session, err);
 
@@ -238,10 +237,39 @@ session_error(struct sp_session *session, enum sp_error err)
       return;
     }
 
-  channel_free(channel);
+  channel_free_all(session);
   session->now_streaming_channel = NULL;
+
+  ap_disconnect(&session->conn);
 }
 
+// Called if an access point disconnects. Will clear current connection and
+// start a flow where the same request will be made to another access point.
+static void
+session_retry(struct sp_session *session)
+{
+  struct sp_channel *channel = session->now_streaming_channel;
+  enum sp_msg_type type = session->msg_type_last;
+  const char *ap_address = ap_address_get(&session->conn);
+  int ret;
+
+  sp_cb.logmsg("Retrying after disconnect (occurred at msg %d)\n", type);
+
+  channel_retry(channel);
+
+  free(session->ap_avoid);
+  session->ap_avoid = strdup(ap_address);
+
+  ap_disconnect(&session->conn);
+
+  // If we were in the middle of a handshake when disconnected we must restart
+  if (msg_is_handshake(type))
+    type = MSG_TYPE_CLIENT_HELLO;
+
+  ret = request_make(type, session);
+  if (ret < 0)
+    session_error(session, ret);
+}
 
 /* ------------------------ Main sequence control --------------------------- */
 
@@ -335,6 +363,15 @@ response_cb(int fd, short what, void *arg)
   if (what == EV_READ)
     {
       ret = evbuffer_read(conn->incoming, fd, -1);
+#ifdef DEBUG_DISCONNECT
+      debug_disconnect_counter++;
+      if (debug_disconnect_counter == 1000)
+	{
+	  sp_cb.logmsg("Simulating a disconnection from the access point (last request type was %d)\n", session->msg_type_last);
+	  ret = 0;
+	}
+#endif
+
       if (ret == 0)
 	RETURN_ERROR(SP_ERR_NOCONNECTION, "The access point disconnected");
       else if (ret < 0)
@@ -349,7 +386,7 @@ response_cb(int fd, short what, void *arg)
       case SP_OK_WAIT: // Incomplete, wait for more data
 	break;
       case SP_OK_DATA:
-        if (channel->is_writing && !channel->file.end_of_file)
+        if (channel->state == SP_CHANNEL_STATE_PLAYING && !channel->file.end_of_file)
 	  session->msg_type_next = MSG_TYPE_CHUNK_REQUEST;
 	if (channel->progress_cb)
 	  channel->progress_cb(channel->audio_fd[0], channel->cb_arg, 4 * channel->file.received_words - SP_OGG_HEADER_LEN, 4 * channel->file.len_words - SP_OGG_HEADER_LEN);
@@ -376,7 +413,10 @@ response_cb(int fd, short what, void *arg)
   return;
 
  error:
-  session_error(session, ret);
+  if (ret == SP_ERR_NOCONNECTION)
+    session_retry(session);
+  else
+    session_error(session, ret);
 }
 
 static int
@@ -384,9 +424,6 @@ relogin(enum sp_msg_type type, struct sp_session *session)
 {
   int ret;
 
-  if (session->msg_type_queued != MSG_TYPE_NONE)
-    RETURN_ERROR(SP_ERR_NOCONNECTION, "Cannot send message, another request is waiting for handshake");
-
   ret = request_make(MSG_TYPE_CLIENT_HELLO, session);
   if (ret < 0)
     RETURN_ERROR(ret, sp_errmsg);
@@ -409,16 +446,16 @@ request_make(enum sp_msg_type type, struct sp_session *session)
   struct sp_conn_callbacks cb = { sp_evbase, response_cb, timeout_cb };
   int ret;
 
+//  sp_cb.logmsg("Making request %d\n", type);
+
   // Make sure the connection is in a state suitable for sending this message
-  ret = ap_connect(type, &cb, session);
+  ret = ap_connect(&session->conn, type, &session->cooldown_ts, session->ap_avoid, &cb, session);
   if (ret == SP_OK_WAIT)
     return relogin(type, session); // Can't proceed right now, the handshake needs to complete first
   else if (ret < 0)
     RETURN_ERROR(ret, sp_errmsg);
 
   ret = msg_make(&msg, type, session);
-  if (type == MSG_TYPE_CLIENT_RESPONSE_ENCRYPTED)
-    memset(session->credentials.password, 0, sizeof(session->credentials.password));
   if (ret < 0)
     RETURN_ERROR(SP_ERR_INVALID, "Error constructing message to Spotify");
 
@@ -436,6 +473,7 @@ request_make(enum sp_msg_type type, struct sp_session *session)
   else
     event_active(session->continue_ev, 0, 0);
 
+  session->msg_type_last = type;
   session->msg_type_next = msg.type_next;
   session->response_handler = msg.response_handler;
 
@@ -464,14 +502,14 @@ track_write(void *arg, int *retval)
     RETURN_ERROR(SP_ERR_NOSESSION, "Cannot play track, no valid session found");
 
   channel = session->now_streaming_channel;
-  if (!channel || !channel->is_allocated)
+  if (!channel || channel->state == SP_CHANNEL_STATE_UNALLOCATED)
     RETURN_ERROR(SP_ERR_INVALID, "No active channel to play, has track been opened?");
 
   channel_play(channel);
 
   ret = request_make(MSG_TYPE_CHUNK_REQUEST, session);
   if (ret < 0)
-    RETURN_ERROR(SP_ERR_NOCONNECTION, "Could not send request for audio chunk");
+    RETURN_ERROR(ret, sp_errmsg);
 
   channel->progress_cb = cmdargs->progress_cb;
   channel->cb_arg = cmdargs->cb_arg;
@@ -497,19 +535,20 @@ track_pause(void *arg, int *retval)
     RETURN_ERROR(SP_ERR_NOSESSION, "Cannot pause track, no valid session found");
 
   channel = session->now_streaming_channel;
-  if (!channel || !channel->is_allocated)
+  if (!channel || channel->state == SP_CHANNEL_STATE_UNALLOCATED)
     RETURN_ERROR(SP_ERR_INVALID, "No active channel to pause, has track been opened?");
 
   // If we are playing we are in the process of downloading a chunk, and in that
   // case we need that to complete before doing anything else with the channel,
   // e.g. reset it as track_close() does.
-  if (!channel->is_writing)
+  if (channel->state != SP_CHANNEL_STATE_PLAYING)
     {
       *retval = 0;
       return COMMAND_END;
     }
 
   channel_pause(channel);
+  session->msg_type_next = MSG_TYPE_NONE;
 
   *retval = 1;
   return COMMAND_PENDING;
@@ -532,9 +571,9 @@ track_seek(void *arg, int *retval)
     RETURN_ERROR(SP_ERR_NOSESSION, "Cannot seek, no valid session found");
 
   channel = session->now_streaming_channel;
-  if (!channel || !channel->is_allocated)
+  if (!channel)
     RETURN_ERROR(SP_ERR_INVALID, "No active channel to seek, has track been opened?");
-  else if (channel->is_writing)
+  else if (channel->state != SP_CHANNEL_STATE_OPENED)
     RETURN_ERROR(SP_ERR_INVALID, "Seeking during playback not currently supported");
 
   // This operation is not safe during chunk downloading because it changes the
@@ -543,7 +582,7 @@ track_seek(void *arg, int *retval)
 
   ret = request_make(MSG_TYPE_CHUNK_REQUEST, session);
   if (ret < 0)
-    RETURN_ERROR(SP_ERR_NOCONNECTION, "Could not send track seek request");
+    RETURN_ERROR(ret, sp_errmsg);
 
   *retval = 1;
   return COMMAND_PENDING;
@@ -612,7 +651,7 @@ media_open(void *arg, int *retval)
   // then the first chunk (incl. headers)
   ret = request_make(type, session);
   if (ret < 0)
-    RETURN_ERROR(SP_ERR_NOCONNECTION, "Could not send media request");
+    RETURN_ERROR(ret, sp_errmsg);
 
   *retval = 1;
   return COMMAND_PENDING;
@@ -801,6 +840,8 @@ librespotc_write(int fd, sp_progress_cb progress_cb, void *cb_arg)
 
   cmdargs = calloc(1, sizeof(struct sp_cmdargs));
 
+  assert(cmdargs);
+
   cmdargs->fd_read     = fd;
   cmdargs->progress_cb = progress_cb;
   cmdargs->cb_arg      = cb_arg;
@@ -907,6 +948,19 @@ librespotc_last_errmsg(void)
   return sp_errmsg ? sp_errmsg : "(no error)";
 }
 
+static void
+system_info_set(struct sp_sysinfo *si_out, struct sp_sysinfo *si_user)
+{
+  memcpy(si_out, si_user, sizeof(struct sp_sysinfo));
+
+  if (si_out->client_name[9] == '\0')
+    snprintf(si_out->client_name, sizeof(si_out->client_name), SP_CLIENT_NAME_DEFAULT);
+  if (si_out->client_version[9] == '\0')
+    snprintf(si_out->client_version, sizeof(si_out->client_version), SP_CLIENT_VERSION_DEFAULT);
+  if (si_out->client_build_id[9] == '\0')
+    snprintf(si_out->client_build_id, sizeof(si_out->client_build_id), SP_CLIENT_BUILD_ID_DEFAULT);
+}
+
 int
 librespotc_init(struct sp_sysinfo *sysinfo, struct sp_callbacks *callbacks)
 {
@@ -918,7 +972,7 @@ librespotc_init(struct sp_sysinfo *sysinfo, struct sp_callbacks *callbacks)
   sp_cb     = *callbacks;
   sp_initialized = true;
 
-  memcpy(&sp_sysinfo, sysinfo, sizeof(struct sp_sysinfo));
+  system_info_set(&sp_sysinfo, sysinfo);
 
   sp_evbase = event_base_new();
   if (!sp_evbase)
