@@ -42,10 +42,6 @@
 # include <pthread_np.h>
 #endif
 
-#include <unistr.h>
-#include <unictype.h>
-#include <uninorm.h>
-
 #include <event2/event.h>
 
 #ifdef HAVE_REGEX_H
@@ -170,10 +166,10 @@ virtual_path_make(char *virtual_path, int virtual_path_len, const char *path)
 
   ret = snprintf(virtual_path, virtual_path_len, "/file:%s", path);
   if ((ret < 0) || (ret >= virtual_path_len))
-  {
-    DPRINTF(E_LOG, L_SCAN, "Virtual path '/file:%s', virtual_path_len exceeded (%d/%d)\n", path, ret, virtual_path_len);
-    return -1;
-  }
+    {
+      DPRINTF(E_LOG, L_SCAN, "Virtual path '/file:%s', virtual_path_len exceeded (%d/%d)\n", path, ret, virtual_path_len);
+      return -1;
+    }
 
   return 0;
 }
@@ -438,6 +434,7 @@ playlist_fill(struct playlist_info *pli, const char *path)
   pli->path  = strdup(path);
   pli->title = strip_extension(filename); // Will alloc
   pli->virtual_path = strip_extension(virtual_path); // Will alloc
+  pli->scan_kind = SCAN_KIND_FILES;
 
   pli->directory_id = get_parent_dir_id(path);
 
@@ -586,6 +583,7 @@ process_regular_file(const char *file, struct stat *sb, int type, int flags, int
   mfi.virtual_path = strdup(virtual_path);
 
   mfi.directory_id = dir_id;
+  mfi.scan_kind = SCAN_KIND_FILES;
 
   if (S_ISFIFO(sb->st_mode))
     {
@@ -699,7 +697,7 @@ process_file(char *file, struct stat *sb, enum file_type file_type, int scan_typ
 
 	DPRINTF(E_LOG, L_SCAN, "Startup rescan triggered, found init-rescan file: %s\n", file);
 
-	library_rescan();
+	library_rescan(0);
 	break;
 
       case FILE_CTRL_METASCAN:
@@ -708,7 +706,7 @@ process_file(char *file, struct stat *sb, enum file_type file_type, int scan_typ
 
 	DPRINTF(E_LOG, L_SCAN, "Meta rescan triggered, found meta-rescan file: %s\n", file);
 
-	library_metarescan();
+	library_metarescan(0);
 	break;
 
       case FILE_CTRL_FULLSCAN:
@@ -816,7 +814,6 @@ process_directory(char *path, int parent_id, int flags)
   if (!dirp)
     {
       DPRINTF(E_LOG, L_SCAN, "Could not open directory %s: %s\n", path, strerror(errno));
-
       return;
     }
 
@@ -824,9 +821,12 @@ process_directory(char *path, int parent_id, int flags)
 
   ret = virtual_path_make(virtual_path, sizeof(virtual_path), path);
   if (ret < 0)
-    return;
+    {
+      closedir(dirp);
+      return;
+    }
 
-  dir_id = db_directory_addorupdate(virtual_path, path, 0, parent_id);
+  dir_id = library_directory_save(virtual_path, path, 0, parent_id, SCAN_KIND_FILES);
   if (dir_id <= 0)
     {
       DPRINTF(E_LOG, L_SCAN, "Insert or update of directory failed '%s'\n", virtual_path);
@@ -853,7 +853,6 @@ process_directory(char *path, int parent_id, int flags)
       if (errno)
 	{
 	  DPRINTF(E_LOG, L_SCAN, "readdir error in %s: %s\n", path, strerror(errno));
-
 	  break;
 	}
 
@@ -879,7 +878,6 @@ process_directory(char *path, int parent_id, int flags)
       if (ret < 0)
 	{
 	  DPRINTF(E_LOG, L_SCAN, "Skipping %s, read_attributes() failed\n", entry);
-
 	  continue;
 	}
 
@@ -916,7 +914,6 @@ process_directory(char *path, int parent_id, int flags)
   if (wi.wd < 0)
     {
       DPRINTF(E_WARN, L_SCAN, "Could not create inotify watch for %s: %s\n", path, strerror(errno));
-
       return;
     }
 
@@ -957,7 +954,7 @@ process_parent_directories(char *path)
       if (ret < 0)
 	return 0;
 
-      dir_id = db_directory_addorupdate(virtual_path, buf, 0, dir_id);
+      dir_id = library_directory_save(virtual_path, buf, 0, dir_id, SCAN_KIND_FILES);
       if (dir_id <= 0)
 	{
 	  DPRINTF(E_LOG, L_SCAN, "Insert or update of directory failed '%s'\n", virtual_path);
@@ -1112,8 +1109,8 @@ static void
 process_inotify_dir(struct watch_info *wi, char *path, struct inotify_event *ie)
 {
   struct watch_enum we;
+  struct watch_info dummy_wi;
   uint32_t rm_wd;
-  char *s;
   int flags = 0;
   int ret;
   int parent_id;
@@ -1204,12 +1201,9 @@ process_inotify_dir(struct watch_info *wi, char *path, struct inotify_event *ie)
       DPRINTF(E_DBG, L_SCAN, "Directory permissions changed (%s): %s\n", wi->path, path);
 
       // Find out if we are already watching the dir (ret will be 0)
-      s = wi->path;
-      wi->path = path;
-      ret = db_watch_get_bypath(wi);
+      ret = db_watch_get_bypath(&dummy_wi, path);
       if (ret == 0)
-	free(wi->path);
-      wi->path = s;
+	free_wi(&dummy_wi, 1);
 
       // We don't use access() or euidaccess() because they don't work with ACL's
       // - this also means we can't check for executable permission, which stat()
@@ -1237,7 +1231,7 @@ process_inotify_dir(struct watch_info *wi, char *path, struct inotify_event *ie)
 	  DPRINTF(E_INFO, L_SCAN, "Directory event, but '%s' already being watched\n", path);
 	}
 
-      if (fd > 0)
+      if (fd >= 0)
 	close(fd);
     }
 
@@ -1321,7 +1315,7 @@ process_inotify_file(struct watch_info *wi, char *path, struct inotify_event *ie
 	  ie->mask |= IN_CLOSE_WRITE;
 	}
 
-      if (fd > 0)
+      if (fd >= 0)
 	close(fd);
     }
 
@@ -1336,7 +1330,7 @@ process_inotify_file(struct watch_info *wi, char *path, struct inotify_event *ie
 	  ret = virtual_path_make(dir_vpath, sizeof(dir_vpath), path);
 	  if (ret >= 0)
 	    {
-	      ptr = strrchr(dir_vpath, '/');
+	      CHECK_NULL(L_SCAN, ptr = strrchr(dir_vpath, '/'));
 	      *ptr = '\0';
 
 	      dir_id = db_directory_id_byvirtualpath(dir_vpath);
@@ -1539,8 +1533,7 @@ inotify_cb(int fd, short event, void *arg)
        * the memory space for ie[1+] contains the name of the file
        * see the inotify documentation
        */
-      wi.wd = ie->wd;
-      ret = db_watch_get_bywd(&wi);
+      ret = db_watch_get_bywd(&wi, ie->wd);
       if (ret < 0)
 	{
 	  if (!(ie->mask & IN_IGNORED))
@@ -1554,30 +1547,30 @@ inotify_cb(int fd, short event, void *arg)
 	  DPRINTF(E_DBG, L_SCAN, "%s deleted or backing filesystem unmounted!\n", wi.path);
 
 	  db_watch_delete_bywd(ie->wd);
-	  free(wi.path);
+	  free_wi(&wi, 1);
 	  continue;
 	}
 
       path[0] = '\0';
 
-      ret = snprintf(path, PATH_MAX, "%s", wi.path);
-      if ((ret < 0) || (ret >= PATH_MAX))
+      ret = snprintf(path, sizeof(path), "%s", wi.path);
+      if ((ret < 0) || (ret >= sizeof(path)))
 	{
 	  DPRINTF(E_LOG, L_SCAN, "Skipping event under %s, PATH_MAX exceeded\n", wi.path);
 
-	  free(wi.path);
+	  free_wi(&wi, 1);
 	  continue;
 	}
 
       if (ie->len > 0)
 	{
-	  namelen = PATH_MAX - ret;
+	  namelen = sizeof(path) - ret;
 	  ret = snprintf(path + ret, namelen, "/%s", ie->name);
 	  if ((ret < 0) || (ret >= namelen))
 	    {
 	      DPRINTF(E_LOG, L_SCAN, "Skipping %s/%s, PATH_MAX exceeded\n", wi.path, ie->name);
 
-	      free(wi.path);
+	      free_wi(&wi, 1);
 	      continue;
 	    }
 	}
@@ -1595,7 +1588,7 @@ inotify_cb(int fd, short event, void *arg)
 #else
 	process_inotify_file_defer(&wi, path, ie);
 #endif
-      free(wi.path);
+      free_wi(&wi, 1);
     }
 
   free(buf);
@@ -1909,7 +1902,7 @@ playlist_add_files(FILE *fp, int pl_id, const char *virtual_path)
 
   if (qp.results > 0)
     {
-      while (((ret = db_query_fetch_file(&qp, &dbmfi)) == 0) && (dbmfi.id))
+      while ((ret = db_query_fetch_file(&dbmfi, &qp)) == 0)
         {
 	  if ((safe_atou32(dbmfi.data_kind, &data_kind) < 0)
 	      || (data_kind == DATA_KIND_PIPE))
@@ -2159,7 +2152,7 @@ filescanner_deinit(void)
 
 struct library_source filescanner =
 {
-  .name = "filescanner",
+  .scan_kind = SCAN_KIND_FILES,
   .disabled = 0,
   .init = filescanner_init,
   .deinit = filescanner_deinit,
